@@ -15,11 +15,15 @@ import responses
 from urlobject import URLObject
 import flask
 from freezegun import freeze_time
-from werkzeug.contrib.fixers import ProxyFix
+from werkzeug.middleware.proxy_fix import ProxyFix
 from flask_dance.consumer import (
-    OAuth2ConsumerBlueprint, oauth_authorized, oauth_error
+    OAuth2ConsumerBlueprint,
+    oauth_authorized,
+    oauth_before_login,
+    oauth_error,
 )
 from flask_dance.consumer.requests import OAuth2Session
+from flask_dance.consumer.storage import MemoryStorage
 
 try:
     import blinker
@@ -29,7 +33,9 @@ requires_blinker = pytest.mark.skipif(not blinker, reason="requires blinker")
 
 
 def make_app(login_url=None, debug=False):
-    blueprint = OAuth2ConsumerBlueprint("test-service", __name__,
+    blueprint = OAuth2ConsumerBlueprint(
+        "test-service",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         scope="admin",
@@ -51,12 +57,14 @@ def make_app(login_url=None, debug=False):
 
     return app, blueprint
 
+
 def test_generate_login_url():
     app, _ = make_app()
     with app.test_request_context("/"):
         login_url = flask.url_for("test-service.login")
         assert login_url == "/login/test-service"
 
+
 def test_override_login_url():
     app, _ = make_app(login_url="/crazy/custom/url")
     with app.test_request_context("/"):
@@ -69,9 +77,7 @@ def test_login_url():
     app, _ = make_app()
     with app.test_client() as client:
         resp = client.get(
-            "/login/test-service",
-            base_url="https://a.b.c",
-            follow_redirects=False,
+            "/login/test-service", base_url="https://a.b.c", follow_redirects=False
         )
         # check that we saved the state in the session
         assert flask.session["test-service_oauth_state"] == "random-string"
@@ -80,7 +86,10 @@ def test_login_url():
     location = URLObject(resp.headers["Location"])
     assert location.without_query() == "https://example.com/oauth/authorize"
     assert location.query_dict["client_id"] == "client_id"
-    assert location.query_dict["redirect_uri"] == "https://a.b.c/login/test-service/authorized"
+    assert (
+        location.query_dict["redirect_uri"]
+        == "https://a.b.c/login/test-service/authorized"
+    )
     assert location.query_dict["scope"] == "admin"
     assert location.query_dict["state"] == "random-string"
 
@@ -88,7 +97,7 @@ def test_login_url():
 @responses.activate
 def test_login_url_forwarded_proto():
     app, _ = make_app()
-    app.wsgi_app = ProxyFix(app.wsgi_app)
+    app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1)
     with app.test_client() as client:
         resp = client.get(
             "/login/test-service",
@@ -99,7 +108,10 @@ def test_login_url_forwarded_proto():
     # check that we redirected the client with a https redirect_uri
     assert resp.status_code == 302
     location = URLObject(resp.headers["Location"])
-    assert location.query_dict["redirect_uri"] == "https://a.b.c/login/test-service/authorized"
+    assert (
+        location.query_dict["redirect_uri"]
+        == "https://a.b.c/login/test-service/authorized"
+    )
 
 
 @responses.activate
@@ -125,13 +137,16 @@ def test_authorized_url():
         # check that we obtained an access token
         assert len(responses.calls) == 1
         request_data = dict(parse_qsl(responses.calls[0].request.body))
-        assert request_data["client_id"] == "client_id"
-        assert request_data["redirect_uri"] == "https://a.b.c/login/test-service/authorized"
-        # check that we stored the access token in the session
         assert (
-            flask.session["test-service_oauth_token"] ==
-            {'access_token': 'foobar', 'scope': ['admin'], 'token_type': 'bearer'}
+            request_data["redirect_uri"]
+            == "https://a.b.c/login/test-service/authorized"
         )
+        # check that we stored the access token in the session
+        assert flask.session["test-service_oauth_token"] == {
+            "access_token": "foobar",
+            "scope": ["admin"],
+            "token_type": "bearer",
+        }
 
 
 def test_authorized_url_no_state():
@@ -157,7 +172,7 @@ def test_authorized_url_behind_proxy():
         body='{"access_token":"foobar","token_type":"bearer","scope":"admin"}',
     )
     app, _ = make_app()
-    app.wsgi_app = ProxyFix(app.wsgi_app)
+    app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1)
     with app.test_client() as client:
         # reset the session before the request
         with client.session_transaction() as sess:
@@ -170,7 +185,10 @@ def test_authorized_url_behind_proxy():
         )
         request_data = dict(parse_qsl(responses.calls[0].request.body))
         # this should be https
-        assert request_data["redirect_uri"] == "https://a.b.c/login/test-service/authorized"
+        assert (
+            request_data["redirect_uri"]
+            == "https://a.b.c/login/test-service/authorized"
+        )
 
 
 def test_authorized_url_invalid_response():
@@ -182,12 +200,16 @@ def test_authorized_url_invalid_response():
         # make the request
         with pytest.raises(MissingCodeError) as missingError:
             client.get(
-                "/login/test-service/authorized?error_code=1349048&error_message=IMUSEFUL",
+                "/login/test-service/authorized?state=random-string&error_code=1349048&error_message=IMUSEFUL",
                 base_url="https://a.b.c",
             )
         match = re.search(r"{[^}]*}", str(missingError.value))
         err_dict = json.loads(match.group(0))
-        assert err_dict == {"error_message": "IMUSEFUL", "error_code": "1349048"}
+        assert err_dict == {
+            "state": "random-string",
+            "error_message": "IMUSEFUL",
+            "error_code": "1349048",
+        }
 
 
 @responses.activate
@@ -215,14 +237,16 @@ def test_authorized_url_token_lifetime():
         # check that we obtained an access token
         assert len(responses.calls) == 1
         request_data = dict(parse_qsl(responses.calls[0].request.body))
-        assert request_data["client_id"] == "client_id"
-        assert request_data["redirect_uri"] == "https://a.b.c/login/test-service/authorized"
+        assert (
+            request_data["redirect_uri"]
+            == "https://a.b.c/login/test-service/authorized"
+        )
         # check that we stored the access token and expiration date in the session
         expected_stored_token = {
-            'access_token': 'foobar',
-            'token_type': 'bearer',
-            'expires_in': 300,
-            'expires_at': 1451649901,
+            "access_token": "foobar",
+            "token_type": "bearer",
+            "expires_in": 300,
+            "expires_at": 1451649901,
         }
         assert flask.session["test-service_oauth_token"] == expected_stored_token
 
@@ -233,11 +257,11 @@ def test_return_expired_token(request):
     time2 = "2016-01-01 12:05:00"  # 299 sec in future
     time3 = "2016-01-01 12:10:01"  # 600 sec in future
     token = {
-        'access_token': 'foobar',
-        'token_type': 'bearer',
-        'expires_in': 300,  # expires in 300 seconds
+        "access_token": "foobar",
+        "token_type": "bearer",
+        "expires_in": 300,  # expires in 300 seconds
     }
-    ctx = app.test_request_context('/')
+    ctx = app.test_request_context("/")
     request.addfinalizer(ctx.pop)
     ctx.push()
 
@@ -286,7 +310,9 @@ def test_redirect_url():
         "https://example.com/oauth/access_token",
         body='{"access_token":"foobar","token_type":"bearer","scope":"admin"}',
     )
-    blueprint = OAuth2ConsumerBlueprint("test-service", __name__,
+    blueprint = OAuth2ConsumerBlueprint(
+        "test-service",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         state="random-string",
@@ -320,7 +346,9 @@ def test_redirect_to():
         "https://example.com/oauth/access_token",
         body='{"access_token":"foobar","token_type":"bearer","scope":"admin"}',
     )
-    blueprint = OAuth2ConsumerBlueprint("test-service", __name__,
+    blueprint = OAuth2ConsumerBlueprint(
+        "test-service",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         state="random-string",
@@ -358,7 +386,9 @@ def test_redirect_fallback():
         "https://example.com/oauth/access_token",
         body='{"access_token":"foobar","token_type":"bearer","scope":"admin"}',
     )
-    blueprint = OAuth2ConsumerBlueprint("test-service", __name__,
+    blueprint = OAuth2ConsumerBlueprint(
+        "test-service",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         state="random-string",
@@ -388,11 +418,44 @@ def test_redirect_fallback():
         assert resp.headers["Location"] == "https://a.b.c/"
 
 
+def test_authorization_required_decorator_allowed():
+    app, blueprint = make_app()
+
+    @app.route("/restricted")
+    @blueprint.session.authorization_required
+    def restricted_view():
+        return "allowed"
+
+    blueprint.storage = MemoryStorage({"access_token": "faketoken"})
+
+    with app.test_client() as client:
+        resp = client.get("/restricted", base_url="https://a.b.c")
+        assert resp.status_code == 200
+        text = resp.get_data(as_text=True)
+        assert text == "allowed"
+
+
+def test_authorization_required_decorator_redirect():
+    app, blueprint = make_app()
+
+    @app.route("/restricted")
+    @blueprint.session.authorization_required
+    def restricted_view():
+        return "allowed"
+
+    with app.test_client() as client:
+        resp = client.get("/restricted", base_url="https://a.b.c")
+        # check that we redirected the client
+        assert resp.status_code == 302
+        assert resp.headers["Location"] == "https://a.b.c/login/test-service"
+
+
 @requires_blinker
 def test_signal_oauth_authorized(request):
     app, bp = make_app()
 
     calls = []
+
     def callback(*args, **kwargs):
         calls.append((args, kwargs))
 
@@ -406,7 +469,7 @@ def test_signal_oauth_authorized(request):
 
         bp.session.fetch_token = mock.Mock(return_value=fake_token)
         resp = client.get(
-            "/login/test-service/authorized?code=secret-code&state=random-string",
+            "/login/test-service/authorized?code=secret-code&state=random-string"
         )
         assert resp.status_code == 302
         # check that we stored the token
@@ -417,11 +480,29 @@ def test_signal_oauth_authorized(request):
     assert calls[0][1] == {"token": fake_token}
 
 
+@requires_blinker
+def test_signal_oauth_before_login(request):
+    app, bp = make_app()
+
+    def callback(*args, **kwargs):
+        del flask.session["user_id"]
+
+    oauth_before_login.connect(callback)
+    request.addfinalizer(lambda: oauth_before_login.disconnect(callback))
+    with app.test_request_context():
+        with app.test_client() as client:
+            flask.session["user_id"] = 1
+            assert flask.session["user_id"] == 1
+            client.get("/login/test-service")
+            assert "user_id" not in flask.session
+
+
 @requires_blinker
 def test_signal_oauth_authorized_abort(request):
     app, bp = make_app()
 
     calls = []
+
     def callback(*args, **kwargs):
         calls.append((args, kwargs))
         return False
@@ -437,7 +518,7 @@ def test_signal_oauth_authorized_abort(request):
         bp.session.fetch_token = mock.Mock(return_value=fake_token)
 
         resp = client.get(
-            "/login/test-service/authorized?code=secret-code&state=random-string",
+            "/login/test-service/authorized?code=secret-code&state=random-string"
         )
         # check that we did NOT store the token
         assert "test-service_oauth_token" not in flask.session
@@ -451,6 +532,7 @@ def test_signal_oauth_authorized_response(request):
     app, bp = make_app()
 
     calls = []
+
     def callback(*args, **kwargs):
         calls.append((args, kwargs))
         return flask.redirect("/url")
@@ -466,7 +548,7 @@ def test_signal_oauth_authorized_response(request):
         bp.session.fetch_token = mock.Mock(return_value=fake_token)
 
         resp = client.get(
-            "/login/test-service/authorized?code=secret-code&state=random-string",
+            "/login/test-service/authorized?code=secret-code&state=random-string"
         )
         assert resp.status_code == 302
         assert resp.headers["Location"] == "http://localhost/url"
@@ -480,7 +562,9 @@ def test_signal_oauth_authorized_response(request):
 @requires_blinker
 def test_signal_sender_oauth_authorized(request):
     app, bp = make_app()
-    bp2 = OAuth2ConsumerBlueprint("test2", __name__,
+    bp2 = OAuth2ConsumerBlueprint(
+        "test2",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         scope="admin",
@@ -493,6 +577,7 @@ def test_signal_sender_oauth_authorized(request):
     app.register_blueprint(bp2, url_prefix="/login")
 
     calls = []
+
     def callback(*args, **kwargs):
         calls.append((args, kwargs))
 
@@ -509,7 +594,7 @@ def test_signal_sender_oauth_authorized(request):
         bp2.session.fetch_token = mock.Mock(return_value=fake_token2)
 
         resp = client.get(
-            "/login/test2/authorized?code=secret-code&state=random-string",
+            "/login/test2/authorized?code=secret-code&state=random-string"
         )
 
     assert len(calls) == 0
@@ -522,7 +607,7 @@ def test_signal_sender_oauth_authorized(request):
         bp2.session.fetch_token = mock.Mock(return_value="test2-token")
 
         resp = client.get(
-            "/login/test-service/authorized?code=secret-code&state=random-string",
+            "/login/test-service/authorized?code=secret-code&state=random-string"
         )
 
     assert len(calls) == 1
@@ -537,7 +622,7 @@ def test_signal_sender_oauth_authorized(request):
         bp2.session.fetch_token = mock.Mock(return_value=fake_token2)
 
         resp = client.get(
-            "/login/test2/authorized?code=secret-code&state=random-string",
+            "/login/test2/authorized?code=secret-code&state=random-string"
         )
 
     assert len(calls) == 1  # unchanged
@@ -548,6 +633,7 @@ def test_signal_oauth_error(request):
     app, bp = make_app()
 
     calls = []
+
     def callback(*args, **kwargs):
         calls.append((args, kwargs))
 
@@ -578,7 +664,9 @@ class CustomOAuth2Session(OAuth2Session):
 
 
 def test_custom_session_class():
-    bp = OAuth2ConsumerBlueprint("test", __name__,
+    bp = OAuth2ConsumerBlueprint(
+        "test",
+        __name__,
         client_id="client_id",
         client_secret="client_secret",
         scope="admin",
